      PROGRAM RAM98

C /**********************************************************************/
C /*	This program computes SST estimates according to the revised      */
C /*	analog method (RAM) described in Waelbroeck et al. [1998].        */
C /*	It is based on the program by Howell (1985, 1989) that implements */
C /*	the modern analog technique (MAT) [Prell, 1985]. The present code */
C /*	was written by C. Waelbroeck (last modification: March 1998).     */
C /*	ORIGINAL REFERENCE:                                               */
C /*	Waelbroeck, C., L. Labeyrie, J.C. Duplessy, J. Guiot, M.          */
C /*    Labracherie, H. Leclaire, and J. Duprat, Improving past sea     */
C /*    surface temperature estimates based on planktonic fossil faunas,*/
C /*    Paleoceanography, 13: 272-283, 1998.		                        */
C /*	CONTACT INFORMATION:                                              */
C /*	Claire Waelbroeck                                                 */
C /*    Laboratoire des Sciences du Climat et de l'Environnement        */
C /*    Domaine du CNRS, bat. 12, 91198 Gif-sur-Yvette, France          */
C /*    Tel: 33-1 69 82 43 27, Fax: 33-1 69 82 35 68                    */
C /*    e-mail: Claire.Waelbroeck@lsce.cnrs-gif.fr                      */
C /**********************************************************************/

      REAL T(2,5000),Q(5000,45),LAT2(2,5000),LONG2(2,5000),
     &     BUGSUM,TM(2),TS(2),MIN(2),MAX(2),TN(2,50000),G,R,SU,SD,DIST,
     &     UNDF,LAT,LON,P(45),MEAN(4,2),SUMSQR(4,2),SUMD(4),HI(4),
     &     LO(4),D(5000),DSML(5000),PP(45),HIGH,LOW,A,QQ(45),MAXVAL(4),
     &     MINVAL(4),SIM(50),SIGMA(4,2)
      INTEGER TAG1,TAG2,N,NCORE1,NCORE2,OPTION,BETA,DEPTH2(5000),II,
     &        NSH,FL(45),NREF,COUNT,DEPTH,MM,F(5000),Z,K,X,FLAG(5000),
     &        NNEW,ZZ,XX(4),JLOOP,IJ,JX,JUMPS,J2,I,J
      CHARACTER*80 SFILE,AFILE,EFILE,OUT1,OUT2,FMAT(3),FMAT2(3),ENVFMT
      CHARACTER*90 NAMES(3)
      CHARACTER*12 CORE,COREID(5000)
      UNDF=-99.999
C /*********************************************************************/
C /*	Read in information from the Input Deck (similar to the Analog   */
C /*    Deck of MAT). The variables are:                               */
C /*      SFILE = subject faunal data file                             */
C /*      AFILE = reference core tops faunal data file                 */
C /*      EFILE = reference core tops environmental data file          */
C /*      OUT1 = complete output file                                  */
C /*      OUT2 = short output file                                     */
C /*      TAG1 = flag to indicate whether geographical coordinates and */
C /*             core depth are to be read in the subject file or not  */
C /*      TAG2 = idem for reference faunal data                        */
C /*      N = number of species                                        */
C /*      NCORE1 = number of samples in the subject data file          */
C /*      NCORE2 = number of core tops in the reference database       */
C /*      OPTION = 1 if the first best analog is not kept to compute   */
C /*               the SST estimate; = 0 otherwise                     */
C /*      BETA = maximum number of modern analogs retained             */
C /*      NAMES = names of the ordered 45 species in standard faunal   */
C /*              data files                                           */
C /*      FMAT = format of subject faunal data                         */
C /*      FMAT2 = format of reference faunal data                      */
C /*      ENVFMT = format of reference environmental data              */
C /*	Then, read in the values from the environmental data file.       */
C /*********************************************************************/
      WRITE(*,1)
1     FORMAT('Please be patient.  The program is executing.')
      OPEN(8,FILE='Input.Deck',STATUS='OLD')
      READ(8,7) SFILE
      READ(8,7) AFILE
      READ(8,7) EFILE
      READ(8,7) OUT1
      READ(8,7) OUT2
      READ(8,5) TAG1,TAG2
5     FORMAT(2I1)
      READ(8,6) N,NCORE1,NCORE2,OPTION,BETA
6     FORMAT(I2,1X,I4,1X,I4,1X,I1,1X,I2)
      READ(8,7) (NAMES(I),I=1,3)
      READ(8,7) (FMAT(I),I=1,3)
      READ(8,7) (FMAT2(I),I=1,3)
      READ(8,7) ENVFMT
7     FORMAT(A80)
      OPEN(10,FILE=SFILE,STATUS='UNKNOWN')
      OPEN(11,FILE=AFILE,STATUS='UNKNOWN')
      OPEN(12,FILE=EFILE,STATUS='UNKNOWN')
      OPEN(13,FILE=OUT1,STATUS='UNKNOWN')
      OPEN(14,FILE=OUT2,STATUS='UNKNOWN')

      OPEN(16,FILE='info',STATUS='UNKNOWN')
      OPEN(17,FILE='stock',STATUS='UNKNOWN')
      OPEN(18,FILE='Faun.tot',STATUS='UNKNOWN')
      OPEN(19,FILE='SST.tot',STATUS='UNKNOWN')

      READ(12,ENVFMT) (T(1,J),T(2,J),J=1,NCORE2)
      IF ((TAG1.LT.0).OR.(TAG1.GT.1).OR.(TAG2.GT.1).OR.(TAG2.LT.0)) THEN
      WRITE(13,8)
8     FORMAT(/'YOU HAVE NOT INDICATED IF THERE ARE HEADER CARDS IN'/
     &'THE ANALOG OR SUBJECT INPUT FILES.  PLEASE CHECK.'/
     &'***** HALT EXECUTION')
      GOTO 5000
      END IF
C /************************************************************************/
C /*  Read in the data from the reference faunal file. The species counts */
C /*  are converted to proportion form. If there is no species in a core  */
C /*  top, a message is written in the output file and the execution is   */
C /*  stopped.                                                            */
C /************************************************************************/
      DO 24 J=1,NCORE2
        IF (TAG2 .EQ. 0) READ(11,FMAT2) COREID(J),(Q(J,L),L=1,N)
        IF (TAG2 .EQ. 1) READ(11,FMAT2) COREID(J),(LAT2(K,J),K=1,2),
     &  (LONG2(K,J),K=1,2),DEPTH2(J),(Q(J,L),L=1,N)
        BUGSUM=0.0
        DO 18 II=1,N
18      BUGSUM=BUGSUM+Q(J,II)
        IF (BUGSUM .EQ. 0.0) THEN
          WRITE(13,20) J
20        FORMAT(/'THERE ARE NO SPECIES IN ANALOG CORE ',I3,'.'/
     &    'PLEASE CHECK YOUR INPUT.'/'***** HALT EXECUTION')
          GOTO 5000
        END IF
        DO 23 II=1,N
          Q(J,II)=Q(J,II)/BUGSUM
23      CONTINUE
24    CONTINUE
C /********************************************************************/
C /*    Set up headers for the output files.                          */
C /********************************************************************/
      WRITE(13,31)
31    FORMAT('YOUR DISSIMILARITY COEFFICIENT IS:')
      WRITE(13,32)
32    FORMAT('     SQUARED CHORD DISTANCE')
      WRITE(14,33)
33    FORMAT('SAMPLE         DISS      SD     T1     SD    T2    SD')

C /********************************************************************/
C /*    INDIRECT APPROACH:                                            */
C /*      G (gamma) = grid step in the environmental space            */
C /*      R = interpolation radius                                    */
C /*      NSH = number of virtual core tops created by the            */
C /*            interpolation procedure                               */
C /*      NREF = number of reference core tops used in creating a     */
C /*             given virtual core top                               */
C /*      FL = # of the virtual core top created by interpolation     */
C /*    File 'info' contains information about the run (e.g., list of */
C /*    the virtual core tops with corresponding environmental        */
C /*    variables, and number of reference core tops used in the      */
C /*    interpolation). The extended database (i.e., reference core   */
C /*    tops + virtual core tops) is written out in files 'Faun.tot'  */
C /*    and 'SST.tot'.                                                */
C /********************************************************************/
      G=0.25
      R=0.3
C /*********************************************************************/
C /*    Create the grid in the environmental space, then compute the   */
C /*    virtual abundances by weighted interpolation within a distance */
C /*    R of the grid nodes.                                           */
C /*********************************************************************/
      DO 8000 K=1,2
        CALL ENVSTAT(T,TM,TS,NCORE2,K,MIN,MAX)
8000  CONTINUE
      CALL GRID(MIN,MAX,TN,NNEW,G)
      WRITE(16,8001)
8001  FORMAT (/'# virt ct     Nber of ref. ct used   Tcold  Twarm')
      NSH=0
      DO 8040 L=1,N
        FL(L)=0
        DO 8030 J=1,NNEW
          SU=0.
          SD=0.
          NREF=0
          DO 8020 I=1,NCORE2
            DIST=0.
            DO K=1,2
c             DIST=DIST+(TN(K,J)-T(K,I))**2/TS(K)**2
              DIST=DIST+(TN(K,J)-T(K,I))**2
            ENDDO
            IF (DIST.LE.(R**2)) THEN
              IF (DIST.LT.0.0001) DIST=0.0001
              IF (ABS(Q(I,L)-UNDF).LT.0.0001) GOTO 8020
              SU=SU+Q(I,L)/DIST
              SD=SD+1/DIST
              NREF=NREF+1
            ENDIF
8020      CONTINUE
          IF (SD.GT.0.00001) THEN
c         IF ((SD.GT.0.00001).AND.(NREF.GT.1)) THEN
            FL(L)=FL(L)+1
            Q(NCORE2+FL(L),L)=SU/SD
            DO K=1,2
              T(K,NCORE2+FL(L))=TN(K,J)
            ENDDO

            IF (L.EQ.1) THEN
             WRITE(16,*) FL(L),NREF,T(1,NCORE2+FL(L)),T(2,NCORE2+FL(L))
             WRITE(17,8025) '----',FL(L),'----'
8025         FORMAT (A4,I4,A4)
8026         FORMAT (A12)
            ENDIF
          ENDIF
          NSH=JMAX0(FL(L),NSH)
8030    CONTINUE
8040  CONTINUE

      PRINT*,'NNEW=',NNEW
      PRINT*,'NSH=',NSH
      WRITE(16,*) 'Number of virtual ct created:',NSH
      REWIND(17)
      DO 8060 J=1,NSH
        READ (17,8026) COREID(NCORE2+J)
8060  CONTINUE
      DO 8070 J=1,NCORE2+NSH
        WRITE(18,FMAT2) COREID(J),(LAT2(K,J),K=1,2),
     &  (LONG2(K,J),K=1,2),DEPTH2(J),(Q(J,L)*100.,L=1,N)
        WRITE(19,8067) COREID(J),(LAT2(K,J),K=1,2),
     &  (LONG2(K,J),K=1,2),T(1,J),T(2,J)
8067    FORMAT (A12,2F4.0,3X,F5.0,F4.0,5X,F5.2,5X,F5.2)
8070  CONTINUE
C /***********************************************************************/
C /*	This is the main loop of the program. Interior sections of the   */
C /*	code will be explained with block comments inside this loop.     */
C /*	Read in the subject faunal data (reference core top data         */
C /*	were read above). Then, initialize counters and convert the      */
C /*	subject data to proportion form as was done for the reference    */
C /*	data. If there is no species in a given sample, a message is     */
C /*	written to the output file and execution is terminated.          */
C /***********************************************************************/
      COUNT=1
      DO 1000 I=1,NCORE1
        IF (TAG1 .EQ. 0) READ(10,FMAT) CORE,(P(L),L=1,N)
        IF (TAG1 .EQ. 1) READ(10,FMAT) CORE,LAT,LON,DEPTH,(P(L),L=1,N)
        DO 46 M=1,4
         DO 45 MM=1,2
          MEAN(M,MM)=0.0
          SUMSQR(M,MM)=0.0
45       CONTINUE
         SUMD(M)=0.0
         HI(M)=-9999.
         LO(M)=9999.
46      CONTINUE
        BUGSUM=0.0
        DO 48 II=1,N
48      BUGSUM=BUGSUM+P(II)
        IF (BUGSUM .EQ. 0.0) THEN
          WRITE(13,50) I
50        FORMAT(/'THERE ARE NO SPECIES IN SUBJECT SAMPLE',I3,'.'/
     &    'PLEASE CHECK YOUR DATA.'/'***** EXECUTION TERMINATED')
          GOTO 5000
        END IF
        DO 54 II=1,N
54      P(II)=P(II)/BUGSUM
C /***********************************************************************/
C /*	Calculate the dissimilarity coefficients for each sample. Note   */
C /*	that a cut-off value of 0.6 is used to lower the cost. Should a  */
C /*	dissimilarity value be higher than 0.6, then it is not included  */
C /*	in the array 'DSML' which is used to sort for the best analogs.  */
C /***********************************************************************/
        X=0
        DO 400 J=1,NCORE2+NSH
          D(J)=0.0
          DSML(J)=0.0
          F(J)=0
          DO 200 K=1,N
            D(J)=D(J)+(SQRT(P(K))-SQRT(Q(J,K)))*
     &      (SQRT(P(K))-SQRT(Q(J,K)))
200       CONTINUE
          IF (D(J).LE.0.6) THEN
            X=X+1
            DSML(X)=D(J)
            F(X)=J
          END IF
400     CONTINUE
C /********************************************************************/
C /*	Write out the subject faunal data (in percent form instead of */
C /*    proportion form).                                             */
C /********************************************************************/
      DO 600 IJ=1,45
600   PP(IJ)=P(IJ)*100.
      WRITE(13,601) COUNT
601   FORMAT(/'********************* START OF SAMPLE NUMBER ',I4,
     &' *********************')
      WRITE(13,602)
602   FORMAT('YOUR INPUT CORE (IN PERCENT FORM) IS:')
      IF (TAG1.EQ.0) WRITE(13,FMAT) CORE,(PP(MM),MM=1,N)
      IF (TAG1.EQ.1) WRITE(13,FMAT) CORE,LAT,LON,DEPTH,(PP(K),K=1,N)
      IF (X .EQ. 0) THEN
        WRITE(13,603)
603     FORMAT('*** NO REASONABLE ANALOGS FOR THIS CORE ***')
        GOTO 1000
      END IF
      IF (TAG2 .EQ. 0) THEN
        WRITE(13,605)
605     FORMAT(/'THE TOP ANALOGS, ACCORDING TO THE DISSIMILARITY',
     &  ' VALUE, ARE:'//
     &  '   CORE                             ENV1   ENV2       DISS')
      ELSE
        WRITE(13,610)
610     FORMAT(/'THE TOP ANALOGS, ACCORDING TO THE DISSIMILARITY',
     &  ' VALUE, ARE:'//
     &  '   CORE         LAT    LON    DEPTH  ENV1   ENV2    DISS')
      END IF
C /***********************************************************************/
C /*	Sort the dissimilarity coefficients from low to high. Pass       */
C /*	through the data each time and find the next lowest value.       */
C /*	  Z = # of the reference core top corresponding to the Jth       */
C /*	      best analog for sample I.                                  */
C /***********************************************************************/
      DO 615 L=1,NCORE2+NSH
615   FLAG(L)=0
      HIGH=200000.0
      LOW=HIGH

      WRITE(16,616) I
616   FORMAT (/'New sample',I4)
      WRITE(16,*) 'Best analogs selected    Diss coeff'
      DO 750 J=1,BETA
        DO 700 JJ=1,X
          IF (J .EQ. 1) THEN
            IF (DSML(JJ) .LT. LOW) THEN
              LOW=DSML(JJ)
              Z=JJ
            END IF
          ELSE IF ((DSML(JJ).GE.LOW).AND.(DSML(JJ).LT.HIGH)) THEN
            JLOOP=J-1
            DO 650 L=1,JLOOP
              IF (FLAG(L).EQ.JJ) GOTO 700
650         CONTINUE
            HIGH=DSML(JJ)
            Z=JJ
          END IF
700     CONTINUE

C /***********************************************************************/
C /*    OPTIMIZED SELECTION OF THE BEST MODERN ANALOGS:                  */
C /*      A (alpha) = threshold value of the dissimilarity increase rate */
C /*      JX = number of best analogs selected                           */
C /***********************************************************************/
        A=0.1
        IF (((LOW.NE.0.).AND.((DSML(Z)-LOW).LE.(A*DSML(Z)))).OR.
     &  (LOW.EQ.0.).OR.(J.LE.(2+OPTION))) THEN
          IF (TAG2 .EQ. 0) THEN
            WRITE(13,702) COREID(F(Z)),T(1,F(Z)),T(2,F(Z)),DSML(Z)
702         FORMAT(A12,21X,2(2X,F5.2),3X,F8.4)
          ELSE
            XX(1)=INT(LAT2(1,F(Z)))
            XX(2)=INT(LAT2(2,F(Z)))
            XX(3)=INT(LONG2(1,F(Z)))
            XX(4)=INT(LONG2(2,F(Z)))
            WRITE(13,710) COREID(F(Z)),(XX(K),K=1,4),
     &      DEPTH2(F(Z)),(T(K,F(Z)),K=1,2),DSML(Z)
710         FORMAT(A12,4I4,2X,I4,2(2X,F5.2),F8.4)
          END IF

C /**********************************************************************/
C /*	Calculate the mean, standard deviation, maximum and minimum of  */
C /*	dissimilarity, depth (when applicable), latitude (when          */
C /*	applicable), and environmental variables for the top-ranked     */
C /*	analog cores. Call subroutines 'SUMUP' and 'STATS', except for  */
C /*	latitude and longitude where no max and min is calculated.      */
C /**********************************************************************/
          DO 720 IJ=1,45
720       QQ(IJ)=Q(F(Z),IJ)
          IF ((OPTION.EQ.1).AND.(J.EQ.1)) THEN
            GOTO 725
          ELSE
            CALL SIMLAR(P,QQ,SIM(J),N)
            CALL SUMUP(MEAN(1,1),SUMSQR(1,1),MAXVAL(1),MINVAL(1),
     &      HI(1),LO(1),DSML(Z),MEAN(1,2),SUMSQR(1,2),SIM(J),
     &      SUMD(1),1)
            CALL SUMUP(MEAN(2,1),SUMSQR(2,1),MAXVAL(2),MINVAL(2),
     &      HI(2),LO(2),T(1,F(Z)),MEAN(2,2),SUMSQR(2,2),SIM(J),
     &      SUMD(2),2)
            CALL SUMUP(MEAN(3,1),SUMSQR(3,1),MAXVAL(3),MINVAL(3),
     &      HI(3),LO(3),T(2,F(Z)),MEAN(3,2),SUMSQR(3,2),SIM(J),
     &      SUMD(3),3)
          ENDIF
725       CONTINUE

          FLAG(J)=Z
          IF (J .GT. 1) THEN
            LOW=HIGH
            HIGH=20000.0
          END IF
          WRITE(16,*) J,'            ',DSML(Z)
          IF (J.EQ.BETA) THEN
            ZZ=1
          ELSE
            ZZ=0
            GOTO 750
          ENDIF
        ENDIF

        IF (OPTION.EQ.1) THEN
          JX=J-2+ZZ
        ELSE
          JX=J-1+ZZ
        ENDIF
        WRITE(16,*) 'Nber of mod. analogs selected:',JX
        IF (OPTION.EQ.1) THEN
          IF (JX.LT.(BETA-1)) JUMPS=JUMPS+1
        ELSE
          IF (JX.LT.BETA) JUMPS=JUMPS+1
        ENDIF
        IF (JX.EQ.2) J2=J2+1

        DO 730 M=1,3
          CALL STATS(MEAN(M,1),SIGMA(M,1),SUMSQR(M,1),JX,MEAN(M,2),
     &    SIGMA(M,2),SUMSQR(M,2),SUMD(M),M)
730     CONTINUE
        GOTO 775
750   CONTINUE
C /**************************************************************************/
C /*	Write out to the complete output file mean, standard deviation,     */
C /*    minimum and maximum of the dissimilarity coefficient, as well as    */
C /*	the environmental variables for the top-ranked analogs. If depth    */
C /*	and latitude are read from the reference data file, write out the   */
C /*	related statistics. Also, a weighted mean and standard deviation    */
C /*    are written out when applicable.                                    */
C /**************************************************************************/
775   WRITE(13,796)
796   FORMAT(/'THE STATISTICS BELOW ARE FOR THE SORTED CORES:'/)
798   WRITE(13,800)
800   FORMAT('          MEAN      SIGMA     MIN       MAX      ',
     &'W-MEAN    W-SIGMA')
      WRITE(13,850) MEAN(1,1),SIGMA(1,1),MINVAL(1),MAXVAL(1)
850   FORMAT('DISS',2X,4F10.4,4X,'-----',5X,'-----')
      WRITE(13,860) MEAN(2,1),SIGMA(2,1),
     &MINVAL(2),MAXVAL(2),MEAN(2,2),SIGMA(2,2)
860   FORMAT('ENV1',2X,6F10.4)
      WRITE(13,865) MEAN(3,1),SIGMA(3,1),
     &MINVAL(3),MAXVAL(3),MEAN(3,2),SIGMA(3,2)
865   FORMAT('ENV2',2X,6F10.4)
C /*************************************************************************/
C /*	Write out to the short output file the mean and standard deviation */
C /*    of the dissimilarity coefficient, environmental variables, depth,  */
C /*	latitude and longitude (when read in).                             */
C /*************************************************************************/
      WRITE(14,980) CORE,(MEAN(K,1),SIGMA(K,1),K=1,3)
980   FORMAT(A12,2(1X,F8.4),4(1X,F5.2))
      COUNT=COUNT+1
1000  CONTINUE
      CLOSE(8)
      CLOSE(10)
      CLOSE(11)
      CLOSE(12)
      CLOSE(13)
      CLOSE(14)
      WRITE(16,4990) BETA-OPTION,JUMPS,JUMPS*100./NCORE1
4990  FORMAT (/'Nber of samples with fewer than ',I2,' best analogs '
     &,'selected:',I4,'  i.e., ',F4.1,'%')
      WRITE(16,4995) J2,J2*100./NCORE1
4995  FORMAT ('Nber of samples with only 2 best analogs selected:',
     &I4,'  i.e., ',F4.1,'%')
5000  STOP
      END

      SUBROUTINE SUMUP(AVG1,SSQR1,MAXX,MINX,BIG,SML,X,AVG2,SSQR2,W,
     &SUMW,KK)
C /*********************************************************************/
C /* This subroutine takes running sums that will be used for mean and */
C /* standard deviation. Also, calculate max and min values.           */
C /* This subroutine also calculates running sums for weighted mean    */
C /* and standard deviation of the environmental data.                 */
C /*********************************************************************/
      REAL AVG1,SSQR1,MAXX,MINX,BIG,SML,X,AVG2,SSQR2,W,SUMW
      INTEGER KK
      AVG1=AVG1+X
      SSQR1=SSQR1+X*X
      MAXX=AMAX1(X,BIG)
      BIG=MAXX
      MINX=AMIN1(X,SML)
      SML=MINX
      IF (KK .NE. 1) THEN
        AVG2=AVG2+X*W
        SSQR2=SSQR2+X*X*W
        SUMW=SUMW+W
      END IF
      RETURN
      END

      SUBROUTINE STATS(AVG1,DEV1,SSQR1,N,AVG2,DEV2,SSQR2,WSUM,KK)
C /************************************************************************/
C /*	This subroutine calculates mean and standard deviation + weighted */
C /*	mean and SD (when applicable), using the output of subroutine     */
C /*	'SUMUP'.                                                          */
C /************************************************************************/
      REAL AVG1,DEV1,SSQR1,DEV2,SSQR2,AVG2,WSUM
      INTEGER KK,N
      IF (N .NE. 1) THEN
        IF ((SSQR1-AVG1*AVG1/N).LE.0.) THEN
          IF (ABS(SSQR1-AVG1*AVG1/N).GT.0.001)
     &    PRINT*,'argument1 < 0',SSQR1-AVG1*AVG1/N
          DEV1=0.0
        ELSE
          DEV1=SQRT((SSQR1-AVG1*AVG1/N)/(N-1))
        ENDIF
      ELSE
        DEV1=0.0
      END IF
      AVG1=AVG1/N
        IF (KK .NE. 1) AVG2=AVG2/WSUM
        IF (N .EQ. 1) DEV2=0.00
        IF ((KK .NE. 1) .AND. (N .NE. 1)) THEN
          IF ((SSQR2-AVG2*AVG2*WSUM).LE.0.) THEN
            IF (ABS(SSQR2-AVG2*AVG2*WSUM).GT.0.001)
     &      PRINT*,'argument2 < 0',SSQR2-AVG2*AVG2*WSUM
            DEV2=0.0
          ELSE
            DEV2=SQRT((SSQR2-AVG2*AVG2*WSUM)/(N-1))
          ENDIF
        ENDIF
      RETURN
      END

      SUBROUTINE SIMLAR(P,Q,SIM,N)
C /**********************************************************************/
C /*	This subroutine calculated similarity coefficients for the      */
C /*	top-ranked analogs. These values are used in weighted averages. */
C /**********************************************************************/
      REAL SIM,P(45),Q(45)
      SIM=0.0
      DO 7000 K=1,N
        SIM=SIM+SQRT(P(K)*Q(K))
7000  CONTINUE
      RETURN
      END

      SUBROUTINE ENVSTAT(X,XM,XS,N,K,XMIN,XMAX)
C /************************************************************************/
C /*    This subroutine computes the arithmetic mean and standard         */
C /*    deviation of the sets of length N in array X(K,N) and puts the    */
C /*    results in XM(K) and XS(K); computes min(K) and max(K).           */
C /************************************************************************/
      REAL X(2,5000),XM(2),XS(2),XMIN(2),XMAX(2)
      INTEGER K,N
      XM(K)=0.0
      XS(K)=0.0
      XMIN(K)=50.
      XMAX(K)=-20.
      DO 7400 I=1,N
        XM(K)=XM(K)+X(K,I)
        XS(K)=XS(K)+X(K,I)**2
7400  CONTINUE
      XM(K)=XM(K)/N
      IF ((XS(K)-XM(K)**2*N).LT.0.) THEN
        PRINT*,'arg. negat.',(XS(K)-XM(K)**2*N)
      ELSE
        XS(K)=SQRT((XS(K)-XM(K)**2*N)/(N-1))
      ENDIF
      DO 7450 I=1,N
        XMAX(K)=AMAX1(XMAX(K),X(K,I))
        XMIN(K)=AMIN1(XMIN(K),X(K,I))
7450  CONTINUE
      RETURN
      END

      SUBROUTINE GRID(XMIN,XMAX,X,N,G)
C /********************************************************************/
C /*    This subroutine establishes the grid in the environmental     */
C /*    parameters space                                              */
C /********************************************************************/
      REAL XMIN(2),XMAX(2),X(2,50000),G
      INTEGER K,N,J,J1,J2,NBR(2)

      DO K=1,2
        NBR(K)=INT((XMAX(K)-XMIN(K))/G)+1
      ENDDO
      J=0
      DO 7550 J1=-1,NBR(1)+1
        DO 7500 J2=-1,NBR(2)+1
            J=J+1
            X(2,J)=XMIN(2)+J2*G
            X(1,J)=XMIN(1)+J1*G
7500    CONTINUE
7550  CONTINUE
      N=(NBR(1)+2)*(NBR(2)+2)
      WRITE(16,*) 'Total nber of grid nodes:',N
      RETURN
      END


